DROP TABLE IF EXISTS filial;
DROP TABLE IF EXISTS produto;
DROP TABLE IF EXISTS tempo;
DROP TABLE IF EXISTS vendas;
 
CREATE TABLE filial (
   chaveFilial SMALLINT NOT NULL,
   nomeFilial VARCHAR(15) UNIQUE,
   cidade VARCHAR(30),
   estado CHAR(2),
   regiao CHAR(2),
   pais VARCHAR(15),
   CONSTRAINT PKFILIAL PRIMARY KEY (chaveFilial)
);
 
CREATE TABLE produto (
   chaveProduto SMALLINT NOT NULL,
   nomeProduto VARCHAR(30) UNIQUE,
   marca VARCHAR(30),
   categoria VARCHAR(30),
   departamento SMALLINT,
   CONSTRAINT PKPRODUTO PRIMARY KEY (chaveProduto)
);
 
CREATE TABLE tempo (
   chaveTempo SMALLINT NOT NULL,
   dia TINYINT,
   mes VARCHAR(15),
   trimestre TINYINT,
   semestre TINYINT,
   ano SMALLINT,
   CONSTRAINT PKTEMPO PRIMARY KEY (chaveTempo)
);
 
CREATE TABLE vendas (
   chaveFilial SMALLINT NOT NULL,
   chaveProduto SMALLINT NOT NULL,
   chaveTempo SMALLINT NOT NULL,
   lucroDolar DECIMAL(10,2),
   unidadesVendidas SMALLINT,
   CONSTRAINT PKVENDAS PRIMARY KEY (chaveFilial, chaveProduto, chaveTempo),
   FOREIGN KEY (chaveFilial) REFERENCES filial(chaveFilial),
   FOREIGN KEY (chaveProduto) REFERENCES produto(chaveProduto),
   FOREIGN KEY (chaveTempo) REFERENCES tempo(chaveTempo)
);
 
INSERT INTO filial VALUES (1, 'Filial 1', 'Sao Carlos', 'SP', 'SE', 'Brasil');
INSERT INTO filial VALUES (2, 'Filial 2', 'Araraquara', 'SP', 'SE', 'Brasil');
INSERT INTO filial VALUES (3, 'Filial 3', 'Recife', 'PE', 'NE', 'Brasil');
INSERT INTO filial VALUES (4, 'Filial 4', 'Ribeirao Preto', 'SP', 'SE', 'Brasil');
INSERT INTO filial VALUES (5, 'Filial 5', 'Jaboatao', 'PE', 'NE', 'Brasil');
 
INSERT INTO produto VALUES (1, 'coca cola 600 ml', 'Coca Cola', 'refrigerante', 1);
 
INSERT INTO tempo VALUES (1, 1, 'janeiro', 1, 1, 2014);
INSERT INTO tempo VALUES (2, 2, 'janeiro', 1, 1, 2014);
INSERT INTO tempo VALUES (3, 1, 'fevereiro', 1, 1, 2014);
INSERT INTO tempo VALUES (4, 2, 'fevereiro', 1, 1, 2014);
INSERT INTO tempo VALUES (5, 1, 'janeiro', 1, 1, 2015);
INSERT INTO tempo VALUES (6, 2, 'janeiro', 1, 1, 2015);
INSERT INTO tempo VALUES (7, 1, 'fevereiro', 1, 1, 2015);
INSERT INTO tempo VALUES (8, 2, 'fevereiro', 1, 1, 2015);
 
INSERT INTO vendas VALUES (1, 1, 1, 5, 10);
INSERT INTO vendas VALUES (1, 1, 2, 10, 20);
INSERT INTO vendas VALUES (1, 1, 3, 1, 2);
INSERT INTO vendas VALUES (1, 1, 7, 13, 26);
INSERT INTO vendas VALUES (1, 1, 8, 7, 14);
INSERT INTO vendas VALUES (2, 1, 1, 7, 14);
INSERT INTO vendas VALUES (2, 1, 2, 3, 6);
INSERT INTO vendas VALUES (2, 1, 5, 15, 30);
INSERT INTO vendas VALUES (2, 1, 8, 4, 8);
INSERT INTO vendas VALUES (3, 1, 2, 5, 10);
INSERT INTO vendas VALUES (3, 1, 5, 9, 18);
INSERT INTO vendas VALUES (3, 1, 6, 1, 2);
INSERT INTO vendas VALUES (3, 1, 7, 25, 50);
INSERT INTO vendas VALUES (4, 1, 1, 1, 2);
INSERT INTO vendas VALUES (4, 1, 2, 12, 24);
INSERT INTO vendas VALUES (4, 1, 3, 8, 16);
INSERT INTO vendas VALUES (4, 1, 4, 6, 12);
INSERT INTO vendas VALUES (4, 1, 5, 5, 10);
INSERT INTO vendas VALUES (5, 1, 4, 1, 2);
INSERT INTO vendas VALUES (5, 1, 5, 10, 20);
INSERT INTO vendas VALUES (5, 1, 6, 12, 24);
INSERT INTO vendas VALUES (5, 1, 7, 3, 6);
INSERT INTO vendas VALUES (5, 1, 8, 4, 8);
 
---- OPERACAO PIVOT ---- REORIENTACAO
 
SELECT chaveTempo, chaveProduto, chaveFilial, lucroDolar, unidadesVendidas
FROM vendas
ORDER BY chaveTempo, chaveProduto, chaveFilial
 
---- OPERACAO PIVOT ---- COM SEMANTICA ADICIONAL PARA MELHOR ENTENDIMENTO DO USUARIO
 
SELECT CONCAT(dia, ' ', mes, ' ', ano) AS data, nomeProduto, nomeFilial, lucroDolar, unidadesVendidas
FROM vendas AS V, tempo AS T, produto AS P, filial AS F
WHERE V.chaveTempo = T.chaveTempo
   AND V.chaveProduto = P.chaveProduto
   AND V.chaveFilial = F.chaveFilial
ORDER BY V.chaveTempo, V.chaveProduto, V.chaveFilial;
 
---- OPERACAO SLICE ---- CORTE PARA UM VALOR FIXO
 
SELECT chaveFilial, chaveProduto, chaveTempo, lucroDolar, unidadesVendidas
FROM vendas
WHERE chaveTempo = 1;
 
---- OPERACAO SLICE ---- COM SEMANTICA ADICIONAL PARA MELHOR ENTENDIMENTO DO USUARIO
 
SELECT CONCAT(nomeFilial, ' localizada em ', cidade) AS "Dados Filial", nomeProduto, CONCAT(dia, ' de ', mes, ' de ', ano) AS Tempo,lucroDolar, unidadesVendidas
FROM vendas V, filial F, tempo T, produto P
WHERE V.chaveFilial = F.chaveFilial
   AND V.chaveTempo = T.chaveTempo
   AND V.chaveProduto = P.chaveProduto
   AND dia = 1 AND mes = 'janeiro' AND ano = 2014;
 
---- OPERACAO DICE ---- CORTE PARA UMA FAIXA DE VALORES
---- COM SEMANTICA ADICIONAL PARA MELHOR ENTENDIMENTO DO USUARIO
 
SELECT nomeFilial AS "Nome da Filial",
   nomeProduto AS "Nome do Produto",
   CONCAT(dia, ' de ', mes, ' de ', ano) AS "Data da Venda",
   lucroDolar AS "Lucro Obtido",
   unidadesVendidas AS "Quantidade Vendida"
FROM vendas V, filial F, tempo T, produto P
WHERE V.chaveFilial = F.chaveFilial
   AND V.chaveProduto = P.chaveProduto
   AND V.chaveTempo = T.chaveTempo
   AND V.chaveTempo BETWEEN 1 AND 2;
 
---- OPERACAO ROLL UP --- DADOS MENOS DETALHADOS
 
SELECT ano, estado, nomeProduto, SUM(lucroDolar) AS "Lucro Anual", SUM(unidadesVendidas) AS "Unidades Vendidas"
FROM vendas V, filial F, tempo T, produto P
WHERE V.chaveFilial = F.chaveFilial
   AND V.chaveTempo = T.chaveTempo
   AND V.chaveProduto = P.chaveProduto
GROUP BY ano, estado, nomeProduto
ORDER BY ano, estado, nomeProduto;
 
---- OPERACAO DRILL-ACROSS
---- TAMBÉM COM SEM NTICA ADICIONAL
 
DROP TABLE IF EXISTS vendasPromocao;
DROP TABLE IF EXISTS promocao;
 
CREATE TABLE promocao (
   chavePromocao SMALLINT NOT NULL,
   nomePromocao VARCHAR(15) UNIQUE,
   caracteristica VARCHAR(30),
   duracao TINYINT,
   CONSTRAINT PKPROMOCAO PRIMARY KEY (chavePromocao)
);
 
 
CREATE TABLE vendasPromocao (
   chavePromocao SMALLINT NOT NULL,
   chaveTempo SMALLINT NOT NULL,
   unidadesVendidas SMALLINT,
   CONSTRAINT PKVENDASPROMOCAO PRIMARY KEY (chavePromocao, chaveTempo),
   FOREIGN KEY (chavePromocao) REFERENCES promocao(chavePromocao),
   FOREIGN KEY (chaveTempo) REFERENCES tempo(chaveTempo)
);
 
INSERT INTO promocao VALUES (1, 'cupom', 'cupom em revista', 1);
 
INSERT INTO vendasPromocao VALUES (1, 1, 20);
INSERT INTO vendasPromocao VALUES (1, 2, 10);
INSERT INTO vendasPromocao VALUES (1, 3, 2);
INSERT INTO vendasPromocao VALUES (1, 4, 1);
INSERT INTO vendasPromocao VALUES (1, 5, 15);
INSERT INTO vendasPromocao VALUES (1, 6, 18);
INSERT INTO vendasPromocao VALUES (1, 7, 13);
INSERT INTO vendasPromocao VALUES (1, 8, 5);
 
SELECT parte1.ano,
   vendaFilial AS "Total vendido na filial",
   vendaPromocao AS "Total vendido em promoção"
FROM
   (SELECT ano, SUM(unidadesVendidas) AS vendaFilial
   FROM vendas VF, tempo TF
   WHERE VF.chaveTempo = TF.chaveTempo
   GROUP BY ano) parte1,
   (SELECT ano, SUM(unidadesVendidas) AS vendaPromocao
   FROM vendasPromocao VP, tempo TP
   WHERE VP.chaveTempo = TP.chaveTempo
   GROUP BY ano) parte2
WHERE parte1.ano = parte2.ano
ORDER BY parte1.ano;
 
---- OPERADOR ROLLUP
---- SEM A SEM NTICA DA RESPOSTA
-- https://www.javatpoint.com/mysql-rollup
 
SELECT chaveFilial, chaveProduto, chaveTempo, SUM(lucroDolar), SUM(unidadesVendidas)
FROM vendas
GROUP BY chaveFilial, chaveProduto, chaveTempo WITH ROLLUP;

---- OPERADOR CUBE
---- SEM A SEM NTICA DA RESPOSTA
-- não tem no MySQL
-- todas as combinações possíveis
 
-- SELECT chaveFilial, chaveProduto, chaveTempo, SUM(lucroDolar) AS "Lucro Obtido", SUM(unidadesVendidas) AS "Unidades Vendidas"
-- FROM vendas
-- GROUP BY CUBE (chaveFilial, chaveProduto, chaveTempo)
-- ORDER BY chaveFilial, chaveProduto, chaveTempo;
 
---- OPERADOR GROUPING SETS ---- MESMO RESULTADO QUE ROLLUP
---- SEM A SEM NTICA DA RESPOSTA
 
-- SELECT chaveFilial, chaveProduto, chaveTempo, SUM(lucroDolar) AS "Lucro Anual", SUM(unidadesVendidas) AS "Unidades Vendidas"
-- FROM vendas
-- GROUP BY GROUPING SETS ((chaveFilial, chaveProduto, chaveTempo), (chaveFilial, chaveProduto), (chaveFilial), ());
 
---- OPERADOR GROUPING SETS ---- MESMO RESULTADO QUE CUBE
---- SEM A SEM NTICA DA RESPOSTA
 
-- SELECT chaveFilial, chaveProduto, chaveTempo, SUM(lucroDolar) AS "Lucro Anual", SUM(unidadesVendidas) AS "Unidades Vendidas"
-- FROM vendas
-- GROUP BY GROUPING SETS ((chaveFilial, chaveProduto, chaveTempo), (chaveFilial, chaveProduto), (chaveProduto,chaveTempo), (chaveFilial, chaveTempo), (chaveFilial), (chaveProduto), (chaveTempo), ());
 
---- EXEMPLO DE WINDOW PARTITIONING
---- Listar a quantidade de unidades vendidas por filial, por produto e por tempo individualmente, e ao mesmo tempo comparar com a soma das quantidades das unidades vendidas por hospital
---- SEM A SEM NTICA DA RESPOSTA
 
SELECT chaveFilial, chaveProduto, chaveTempo,
unidadesVendidas "Vendas por Filial",
SUM(unidadesVendidas) OVER (PARTITION BY chaveFilial) AS "Total Unidades Vendidas Filial"
FROM vendas
ORDER BY chaveFilial, chaveProduto, chaveTempo;
 
---- COM A SEM NTICA DA RESPOSTA
 
SELECT nomeFilial, nomeProduto, CONCAT(dia, ' de ', mes, ' de ', ano) AS "Data da Venda",
   unidadesVendidas AS "Vendas por Filial",
   SUM(unidadesVendidas) OVER (PARTITION BY V.chaveFilial) AS "Total Unidades Vendidas Filial"
FROM vendas V, filial F, tempo T, produto P
WHERE V.chaveFilial = F.chaveFilial
   AND V.chaveProduto = P.chaveProduto
   AND V.chaveTempo = T.chaveTempo
ORDER BY V.chaveFilial, V.chaveProduto, V.chaveTempo;
 
---- EXEMPLO DE WINDOW ORDERING
---- Listar a quantidade de unidades vendidas por filial, por produto e por tempo individualmente, e ao mesmo tempo comparar com a soma das quantidades das unidades vendidas por hospital
---- Acrescentar a classificação das tuplas.
 
---- SEM A SEM NTICA DA RESPOSTA
 
SELECT chaveFilial, chaveProduto, chaveTempo,
   unidadesVendidas AS VendasPorFilial,
   SUM(unidadesVendidas) OVER (PARTITION BY chaveFilial) AS "Total Unidades Vendidas Filial",
   ROW_NUMBER() OVER (PARTITION BY chaveFilial ORDER BY unidadesVendidas, chaveFilial, chaveProduto, chaveTempo) AS classifica
FROM vendas
ORDER BY chaveFilial, chaveProduto, VendasPorFilial;
 
---- COM A SEM NTICA DA RESPOSTA
 
SELECT nomeFilial AS "Nome da Filial",
   nomeProduto AS "Nome do Produto",
   CONCAT(dia, ' de ', mes, ' de ', ano) AS "Data da Venda",
   unidadesVendidas AS VendasPorFilial,
   SUM(unidadesVendidas) OVER (PARTITION BY V.chaveFilial) AS "Total Unidades Vendidas Filial",
   ROW_NUMBER() OVER (PARTITION BY V.chaveFilial ORDER BY unidadesVendidas, V.chaveFilial, V.chaveProduto, V.chaveTempo) AS classifica
FROM vendas V, filial F, produto P, tempo T
WHERE V.chaveFilial = F.chaveFilial
AND V.chaveProduto = P.chaveProduto
AND V.chaveTempo = T.chaveTempo
ORDER BY V.chaveFilial, V.chaveProduto, VendasPorFilial;
 
 
--- DIFERENÇA ENTRE ROW_NUMBER, RANK() E DENSE_RANK
 
SELECT chaveFilial, chaveProduto, chaveTempo,
   unidadesVendidas AS "Vendas por Filial",
   SUM(unidadesVendidas) OVER (PARTITION BY chaveFilial) AS "Total Unidades Vendidas Filial",
   ROW_NUMBER() OVER (ORDER BY unidadesVendidas) AS classifica
FROM vendas
ORDER BY classifica, chaveFilial, chaveProduto, chaveTempo;
 
SELECT chaveFilial, chaveProduto, chaveTempo,
   unidadesVendidas AS "Vendas por Filial",
   SUM(unidadesVendidas) OVER (PARTITION BY chaveFilial) AS "Total Unidades Vendidas Filial",
   RANK() OVER (ORDER BY unidadesVendidas) AS classifica
FROM vendas
ORDER BY classifica, chaveFilial, chaveProduto, chaveTempo;
 
SELECT chaveFilial, chaveProduto, chaveTempo,
   unidadesVendidas AS "Vendas por Filial",
   SUM(unidadesVendidas) OVER (PARTITION BY chaveFilial) AS "Total Unidades Vendidas Filial",
   DENSE_RANK() OVER (ORDER BY unidadesVendidas) AS classifica
   FROM vendas
ORDER BY classifica, chaveFilial, chaveProduto, chaveTempo;
 
